/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die spezielle Umsetzung der Anzeige-Informationen fr das WTA-C-Zeichen in BW.
 * Dieses C-Zeichen kann mit Codes und mit Wechseltext angesteuert werden (Aussenanlage TLS 2002 und hher).
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.3 $ / $Date: 2012/06/29 10:23:00 $ / ($Author: Drapp $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTlsBw1CZeichen extends AnzeigeTypUmsetzungTlsDefault
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();

  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst Liste mit Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   * @param listeAnzeigeStatus Liste mit Status Methoden fr alle Anzeigen.
   */
  public AnzeigeTypUmsetzungTlsBw1CZeichen(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, HashMap listeAnzeigeEigenschaftIst, HashMap listeAnzeigeStatus)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, listeAnzeigeEigenschaftIst, listeAnzeigeStatus);
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps und speziellen Anzeige-Informationen.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param kannFreieGrafik <code>true</code>, wenn Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   * @param kannFreiText <code>true</code>, wenn Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public AnzeigeTypUmsetzungTlsBw1CZeichen(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, boolean kannFreieGrafik, boolean kannFreiText)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, kannFreieGrafik, kannFreiText);
  }
  
  /**
   * Abarbeitung des TLS-WvzStellZustand nach TLS 2002.
   * 
   * @param resultData Attributgruppe atg.tlsWzgStellZustand.
   */
  public Data tlsWzgStellZustand(ResultData resultData)
  {
    SystemObject object = resultData.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsDes().get(object.getPid());

    if (anzeigeAktor == null)
      return null;
    
    String anzeigePid = anzeigeAktor.getAnzeige().getPid();
    
    Data dataAnzeigeEigenschaftIst = (Data)getListeAnzeigeEigenschaftIst().get(anzeigePid);
    
    m_debug.finer("tlsWzgStellZustand DE " + object.getPid() + "\nAnzeige " + anzeigePid + "\nresult " + resultData + "\nData " + dataAnzeigeEigenschaftIst);

    if (resultData.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeEigenschaftIstInitialisieren(dataAnzeigeEigenschaftIst, resultData.getDataTime(), statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (resultData.hasData()) // Daten vorhanden
      {
        Data data = resultData.getData();

        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();

        String prinzip = data.getScaledValue("AnzeigePrinzip").getValueText();
        
        if ((prinzip.equals("c")) || (prinzip.equals("d")))
        {
          if (data.getArray("StellCode").getLength() == 1)
          {
            int code = data.getArray("StellCode").getItem(0).getScaledValue("Code").intValue();

            if (code == 62) // Workaround fr Konfigurationsfehler
            	code = 9;
            
            m_debug.finest("Code " + code);

            if (code == 0) // freie TextDefinition
            {
              if (data.getArray("TextDefinition").getLength() < 7) // Wechseltext der nur aus Formatierung besteht als 'aus' annehmen
              {
              	code = -1; 
              	
                HashMap mapWvzInhalt = null;
                
                mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsCode();
                
                WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(code);
                
                if (wvzInhaltUmsetzungTls != null)
                {
                  dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setText(wvzInhaltUmsetzungTls.getPidWvzInhalt());
                }
                
                tlsWzgWechselTextBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
                
                dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));

                // Wechseltextinformationen lschen
                
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("Wechseltext").setSystemObject(null);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
              }
              else // Wechseltext bestimmen
              {
                // Ergebnis lschen    
                
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("Wechseltext").setSystemObject(null);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
              
                String zeichenfolge = "";
  
                for (int i = 0; i < data.getArray("TextDefinition").getLength(); i++)
                {
                  zeichenfolge += data.getArray("TextDefinition").getItem(i).getTextValue("Text").getValueText();
                }
                
                int laengeInput = zeichenfolge.length();
                int laengeText = laengeInput-6;
          
                if (laengeInput >= 6)
                {

                  // Font aus Rckmeldung bestimmen, Grundfont nicht benutzen                  
//                  String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, zeichenfolge.codePointAt(0), pidGrundFontUmsetzungTls);
                  
                  int codeTextAusrichtung = zeichenfolge.codePointAt(0);
                  
                  // fr Tests Font ber ASCII 1, 2
                  
                  if (codeTextAusrichtung == 49)
                  {
                    codeTextAusrichtung = 128;
                  }
                  else if (codeTextAusrichtung == 50)
                  {
                    codeTextAusrichtung = 129;
                  }
                  
                  String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, codeTextAusrichtung, null);
          
                  FontUmsetzungTls fontUmsetzungTls = null;
      
                  if (pidFontUmsetzungTls != null)
                  {
                    HashMap mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
              
                    if (mapFont != null)
                      fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(pidFontUmsetzungTls);
                  }
      
                  HashMap mapZeichen = null;
            
                  if (fontUmsetzungTls != null)
                    mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsCode();
      
                  // hier mit Fehler abbrechen, wenn kein Font bestimmt !!      
                  
                  if ((mapZeichen == null) || (mapZeichen.size() == 0))
                  {
                    tlsWzgWechselTextBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
                    dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));

                    m_debug.warning(object.getPid() + "Font ber Code " + codeTextAusrichtung + " nicht bestimmbar");

                    return dataAnzeigeEigenschaftIst;
                  }
                  
                  // Pid Font ber Umsetzung bestimmen
          
                  String pidFont = null;
            
                  if (fontUmsetzungTls != null)
                  {
                    pidFont = fontUmsetzungTls.getPidFont();
                  }
              
                  if (laengeText < 0)
                  {
                    laengeText = 0;
        
                    m_debug.warning(object.getPid() + "Wechseltext zu kurz (6 + Text) '" + zeichenfolge + "'");
                
                    return null;
                  }
              
                  if (pidFont != null)
                  {
                    dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setText(pidFont);
                  }

                  // Textausrichtung aus Rckmeldung bestimmen
      
                  codeTextAusrichtung = zeichenfolge.codePointAt(1);
                  
                  // fr Tests Ausrichtung ber ASCII 1, 2, 3
                  
                  if (codeTextAusrichtung == 49)
                  {
                    codeTextAusrichtung = 133;
                  }
                  else if (codeTextAusrichtung == 50)
                  {
                    codeTextAusrichtung = 134;
                  }
                  else if (codeTextAusrichtung == 51)
                  {
                    codeTextAusrichtung = 135;
                  }

                  String textAusrichtung = textAusrichtungAusTlsBestimmen(anzeigeTypUmsetzungTls, codeTextAusrichtung);
              
                  dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").setText(textAusrichtung);
            
                  // Text        
            
                  dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(laengeText);
      
                  for (int i = 6; i < zeichenfolge.length(); i++)
                  {
                    m_debug.finest("Zeichen " + i + " " + zeichenfolge.codePointAt(i));
          
                    int codeZeichen = zeichenfolge.codePointAt(i);
          
                    ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(codeZeichen);
                  
                    if (zeichenUmsetzungTls != null)
                      dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setText(zeichenUmsetzungTls.getPidZeichen());
                    else
                      dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setSystemObject(null);
                  }
                  
                  dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));
                }
                else // fehlerhafte Lnge, wenn grer als 0, wird mindestens 6 erwartet
                {
                  dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
                }
                
                tlsWzgStellZustandBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
              }
            }
            else // code != 0
            {
              HashMap mapWvzInhalt = null;
              
              mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsCode();
              
              WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(code);
              
              if (wvzInhaltUmsetzungTls != null)
              {
                bearbeiteWvzCodePlausibilitaet(anzeigeAktor, object, code, true, resultData.getDataTime());

                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setText(wvzInhaltUmsetzungTls.getPidWvzInhalt());
                dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));
              }
              else
              {
                bearbeiteWvzCodePlausibilitaet(anzeigeAktor, object, code, false, resultData.getDataTime());

                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
                dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
              }
              
              tlsWzgWechselTextBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
              
              // Wechseltextinformationen lschen
              
              dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("Wechseltext").setSystemObject(null);
              dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
              dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
              dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
            }
          }
          else
          {
            tlsWzgWechselTextBlinken(null, dataAnzeigeEigenschaftIst);

            dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
          }
        }
      }
      else // keine Daten vorhanden
      {
        tlsWzgWechselTextBlinken(null, dataAnzeigeEigenschaftIst);

        dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
      }
    }
    
    return dataAnzeigeEigenschaftIst;
  }

  /**
   * Abarbeitung der Befehle aus dem Modell AnzeigenGlobal.
   * 
   * @param resultData AttributGruppe atg.anzeigeEigenschaft.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeEigenschaftSoll(ResultData resultData, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;
    
    SystemObject objekt = resultData.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsAnzeige().get(objekt.getPid());

    if (anzeigeAktor == null)
      return null;
    
    if (resultData.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      return null;
    }
    else // Quelle vorhanden
    {
      if (resultData.hasData()) // Daten vorhanden
      {
        Data data = resultData.getData();
    
        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();
    
        long zeitpunkt = data.getTimeValue("SollZeitpunkt").getMillis();

        // BefehlsArt prfen
        
        int befehlsArt = AnzeigeSollBefehlsArt.eigenschaftBefehlsArtBestimmen(data);
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt)
        {
          // Stellcode-Ansteuerung ok
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Grafik)
        {
          if (isKannFreieGrafik()) // Grafik-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Grafik-Ansteuerung");
            
            return null;
          }
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext)
        {
          if (isKannFreiText()) // Wechseltext-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Wechseltext-Ansteuerung");
            
            return null;
          }
        }
        else
        {
          m_debug.error("ATG fr Anzeige " + resultData.getObject().getPid() + " beinhaltet keinen Befehl");
          
          return null;
        }
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt) // reine Stellcode-Ansteuerung
        {
          HashMap mapWvzInhalt = null;
          
          ConfigurationObject objektAnzeigeInhalt = (ConfigurationObject)data.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").getSystemObject();
          
          AttributeGroup atgWvzInhalt = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wvzInhalt");
          
          Data dataWvzInhalt = objektAnzeigeInhalt.getConfigurationData(atgWvzInhalt);
          String anzeigeInhalt = dataWvzInhalt.getTextValue("Bildinhalt").getValueText();
          
          m_debug.finest("AnzeigeInhalt " + anzeigeInhalt);

          // DEs bestimmen
          
          int anzahlDes = anzeigeAktor.getDes().size();
          
          if (anzahlDes == 1)
          {
            mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsWvzInhalt();
            
            WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(anzeigeInhalt);
  
            int code = -1;
            
            if (wvzInhaltUmsetzungTls != null)
            {
              code = wvzInhaltUmsetzungTls.getCode();
              
              if (code == 9) // Workaround fr Konfigurationsfehler
              	code = 62;
            }
            else
            {
              m_debug.error("Objekt " + resultData.getObject().getPid() + " WvzInhalt " + anzeigeInhalt + " nicht gefunden");
            }
            
            Set set = anzeigeAktor.getDes().keySet();
            
            Iterator it = set.iterator();
            
            String key = (String)it.next();
            
            SystemObject de = anzeigeAktor.getDes().get(key);
            
            int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

            for (int j = 0; j < anzahlGlobalUmsetzung; j++)
            {
              GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

              for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
              {
                GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                
                String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();

                if (asp.length() > 0)
                {
                  if ((atg.equals("atg.tlsWzgWvzStellZustand")) ||
                      (atg.equals("atg.tlsWzgStellZustand")))
                  {
                    AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                    Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                    
                    Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                    if (atg.equals("atg.tlsWzgWvzStellZustand"))
                    {
                      dataErgebnis.getUnscaledValue("WvzCode").set(code);

                      // BlinkenStatus
                      
                      String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                      dataErgebnis.getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
                      
                      // BlinkenDauer
                      
                      if (blinkenStatus.equals("ein"))
                      {
                        if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                        {
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                        }
                        else
                        {
                          double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                          if (blinkenDauerAnzeige < 1.0)
                          {
                            blinkenDauerAnzeige = 1.0;
                          }
                          if (blinkenDauerAnzeige > 3.0)
                          {
                            blinkenDauerAnzeige = 3.0;
                          }
                          int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                          
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                        }
                      }
                      else
                      {
                        dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                      }
                      
                      if (zeitpunkt < ((new Date()).getTime()-600000000))
                      {
                        zeitpunkt = zeitpunktUebergeordnet.longValue();
                      }
                      
                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    }
                    
                    if (atg.equals("atg.tlsWzgStellZustand"))
                    {
                      dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                      
                      if ((anzeigePrinzip.equals("a")) || (anzeigePrinzip.equals("b")))
                      {
                        dataErgebnis.getArray("WVZAnzeige").setLength(1);
                        dataErgebnis.getArray("StellCode").setLength(0);
                      }
                      else
                      {
                        dataErgebnis.getArray("WVZAnzeige").setLength(0);
                        dataErgebnis.getArray("StellCode").setLength(1);
                      }
                      
                      dataErgebnis.getArray("TextDefinition").setLength(0);
                      dataErgebnis.getArray("GruppenDefinition").setLength(0);
                      
                      if ((anzeigePrinzip.equals("a")) || (anzeigePrinzip.equals("b")))
                      {
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getUnscaledValue("Code").set(code);
  
                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
  
                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }
  
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");
                      }
                      else
                      {
                        dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(code);
                        
                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
  
                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }
  
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");
                        
                        if (code == -1) // 'aus', Sonderfall bei WTA-C-Zeichen in BW
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(0);

                          dataErgebnis.getArray("TextDefinition").setLength(6);

                          int codeFont = fontUmsetzungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, anzeigeTypUmsetzungTls.getPidGrundFont());
                          int codeTextAusrichtung = textAusrichtungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, anzeigeTypUmsetzungTls.getGrundTextAusrichtung());
                          
//                          System.out.println("codeFont " + codeFont + " (" + anzeigeTypUmsetzungTls.getPidGrundFont() + ")");
//                          System.out.println("codeTextAusrichtung " + codeTextAusrichtung + " (" + anzeigeTypUmsetzungTls.getGrundTextAusrichtung() + ")");
                          
                          dataErgebnis.getArray("TextDefinition").getItem(0).getTextValue("Text").setText(String.format("%c", codeFont).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(1).getTextValue("Text").setText(String.format("%c", codeTextAusrichtung).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(2).getTextValue("Text").setText(String.format("%c", 0).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(3).getTextValue("Text").setText(String.format("%c", 0).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(4).getTextValue("Text").setText(String.format("%c", 0).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(5).getTextValue("Text").setText(String.format("%c", 0).toString());                
                        }
                      }
                      
                      if (zeitpunkt < ((new Date()).getTime()-600000000))
                      {
                        zeitpunkt = zeitpunktUebergeordnet.longValue();
                      }
                      
                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    }
                    
                    break;
                  }
                }
              }
            }
          }
        }
        else
        {
          if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext) // reine Textansteuerung
          {
            ConfigurationObject wechseltext = (ConfigurationObject)data.getItem("Eigenschaft").getReferenceValue("Wechseltext").getSystemObject();

            Data dataWechseltext = null;
            
            if (wechseltext != null) // konfigurierter Wechseltext
            {
              AttributeGroup atgWechseltext = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wechseltext");
              dataWechseltext = wechseltext.getConfigurationData(atgWechseltext);
            }
            else // dynamischer Wechseltext
            {
              dataWechseltext = data.getItem("Eigenschaft").getItem("DynamischerWechseltext");
            }
            
            String pidFont = dataWechseltext.getReferenceValue("Font").getValueText();
            int codeFont = fontUmsetzungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidFont);

            String pidTextAusrichtung = dataWechseltext.getScaledValue("TextAusrichtung").getValueText();
            int codeTextAusrichtung = textAusrichtungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidTextAusrichtung);

            HashMap mapZeichen = null;
            
            HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
            
            if ((pidFont != null) && (mapFont != null))
            {
              TreeSet<String> treeset = new TreeSet<String>();
              treeset.addAll(mapFont.keySet());
            
              Iterator iterator = treeset.iterator();
            
              FontUmsetzungTls fontUmsetzungTls;
              
              while (iterator.hasNext())
              {
                fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
                
                if (fontUmsetzungTls.getPidFont().equals(pidFont))
                {
                  mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsZeichen();
                  
                  break;
                }
              }
            }

            // DEs bestimmen
              
            int anzahlDes = anzeigeAktor.getDes().size();
            
            if (anzahlDes == 1) // Wechseltext nur mit einer DE
            {
              Set set = anzeigeAktor.getDes().keySet();
              
              Iterator it = set.iterator();
              
              String key = (String)it.next();
              
              SystemObject de = anzeigeAktor.getDes().get(key);
              
              int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

              for (int j = 0; j < anzahlGlobalUmsetzung; j++)
              {
                GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

                for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
                {
                  GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                  
                  String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                  String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                  String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();

                  if (asp.length() > 0)
                  {
                    if ((atg.equals("atg.tlsWzgWechselText")) ||
                        (atg.equals("atg.tlsWzgStellZustand")))
                    {
                      AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                      Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                      
                      Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                      if (atg.equals("atg.tlsWzgWechselText"))
                      {
                        StringBuffer befehl = new StringBuffer();
                        
                        befehl.append(String.format("%c", codeFont));
                        befehl.append(String.format("%c", codeTextAusrichtung));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));

                        ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                        for (int i = 0; i < text.length; i++)
                        {
                          String pid = text[i].getSystemObject().getPid();
                          
                          if (mapZeichen == null)
                          {
                            continue;
                          }
                          
                          ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                          
                          if (zeichenUmsetzungTls != null)
                          {
                            befehl.append(String.format("%c", zeichenUmsetzungTls.getZeichencode()));
                          }
                          else
                          {
                            befehl.append(String.format("%c", 32)); // Leerzeichen als default
                          }
                        }

                        dataErgebnis.getTextValue("WVZTextZeichen").setText(befehl.toString());

                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
                        
                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }

                        if (zeitpunkt < ((new Date()).getTime()-600000000))
                        {
                          zeitpunkt = zeitpunktUebergeordnet.longValue();
                        }
                        
                        ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                      }
                      
                      if (atg.equals("atg.tlsWzgStellZustand"))
                      {
                        dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                        dataErgebnis.getArray("WVZAnzeige").setLength(0);
                        dataErgebnis.getArray("StellCode").setLength(1);
                        dataErgebnis.getArray("TextDefinition").setLength(0);
                        dataErgebnis.getArray("GruppenDefinition").setLength(0);

                        dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(0);

                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);

                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }

                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");

                        ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                        dataErgebnis.getArray("TextDefinition").setLength(6 + text.length);

                        int index = 0;
                        
//                        System.out.println("codeFont " + codeFont);
//                        System.out.println("codeTextAusrichtung " + codeTextAusrichtung);
                        
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeFont).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeTextAusrichtung).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());

                        int zeichen;
                        
                        for (int i = 0; i < text.length; i++)
                        {
                          String pid = text[i].getSystemObject().getPid();
                          
                          zeichen = 32; // Leerzeichen als default
                          
                          if (mapZeichen != null)
                          {
                            ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                            
                            if (zeichenUmsetzungTls != null)
                            {
                              zeichen = zeichenUmsetzungTls.getZeichencode();
                            }
                            else
                            {
                              zeichen = 32; // Leerzeichen als default
                            }
                          }

//                          System.out.println("- zeichen " + zeichen);
                          
                          dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", zeichen).toString());
                        }

                        if (zeitpunkt < ((new Date()).getTime()-600000000))
                        {
                          zeitpunkt = zeitpunktUebergeordnet.longValue();
                        }
                        
                        ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                      }
                      
                      break;
                    }
                  }
                }
              }
            }
            else // anzahlDes != 1
            {
              m_debug.error("Wechseltext nur mit einer DE mglich");
            }
          }
        }
      } 
      else // keine Daten vorhanden
      {
        return null;
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung der Text-Ausrichtung, kann sie nicht bestimmt werden, wird die Text-Ausrichtung 'Undefiniert' zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param code Zeichen aus Text von KExTls.
   * @return Text-Ausrichtung (default 'Undefiniert').
   */
  protected String textAusrichtungAusTlsBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int code)
  {
    String ergebnis = "Undefiniert";

    HashMap<String, TextAusrichtungUmsetzungTls> mapTextAusrichtung = anzeigeTypUmsetzungTls.getListeTextAusrichtungUmsetzungTls();
    
    if (mapTextAusrichtung != null)
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapTextAusrichtung.keySet());
    
      Iterator iterator = treeset.iterator();
    
      TextAusrichtungUmsetzungTls textAusrichtungUmsetzungTls;
      
      while (iterator.hasNext())
      {
        textAusrichtungUmsetzungTls = (TextAusrichtungUmsetzungTls)mapTextAusrichtung.get(iterator.next());
        
        if (code == textAusrichtungUmsetzungTls.getCodeTextAusrichtung())
        {
          ergebnis = textAusrichtungUmsetzungTls.getTextAusrichtung();

          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung des Code der Text-Ausrichtung, kann sie nicht bestimmt werden, wird Code 133 zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param textAusrichtung Pid der TextAusrichtung.
   * @return Code der TextAusrichtung (default 133).
   */
  protected int textAusrichtungAusAnzeigeBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, String textAusrichtung)
  {
    int ergebnis = 133; // Linksbndig als Default

    HashMap<String, TextAusrichtungUmsetzungTls> mapTextAusrichtung = anzeigeTypUmsetzungTls.getListeTextAusrichtungUmsetzungTls();
    
    if ((textAusrichtung != null) && (mapTextAusrichtung != null))
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapTextAusrichtung.keySet());
    
      Iterator iterator = treeset.iterator();
    
      TextAusrichtungUmsetzungTls textAusrichtungUmsetzungTls;
      
      while (iterator.hasNext())
      {
        textAusrichtungUmsetzungTls = (TextAusrichtungUmsetzungTls)mapTextAusrichtung.get(iterator.next());
        
        if (textAusrichtungUmsetzungTls.getTextAusrichtung().equals(textAusrichtung))
        {
          ergebnis = textAusrichtungUmsetzungTls.getCodeTextAusrichtung();

          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung der Pid der FontUmsetzungTls, kann er nicht bestimmt werden, wird die Pid des GrundFontUmsetzungTls zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param code Zeichen aus Text von KExTls.
   * @param pidGrundFontUmsetzungTls Pid des GrundFonts der FontUmsetzungTls.
   * @return Pid der FontUmsetzungTls.
   */
  protected String fontUmsetzungAusTlsBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int code, String pidGrundFontUmsetzungTls)
  {
    String ergebnis = pidGrundFontUmsetzungTls;
    
    HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
    
    if (mapFont != null)
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapFont.keySet());
    
      Iterator iterator = treeset.iterator();
    
      FontUmsetzungTls fontUmsetzungTls;
      
      while (iterator.hasNext())
      {
        fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
        
        if (code == fontUmsetzungTls.getCodeFont())
        {
          ergebnis = fontUmsetzungTls.getPid();
          
          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung des Code des Fontaus der FontUmsetzungTls, kann er nicht bestimmt werden, wird der Code 128 zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param pidFontOderPidFontUmsetzungTls Pid des Font oder Pid der FontUmsetzungTls.
   * @return Code des Font (default 128).
   */
  protected int fontUmsetzungAusAnzeigeBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, String pidFontOderPidFontUmsetzungTls)
  {
    int ergebnis = 128; // Mittelschrift als Default

    HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
    
    if ((pidFontOderPidFontUmsetzungTls != null) && (mapFont != null))
    {
      FontUmsetzungTls fontUmsetzungTls;

      fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(pidFontOderPidFontUmsetzungTls);
      
      if (fontUmsetzungTls != null)
      {
        ergebnis = fontUmsetzungTls.getCodeFont();
        
        return ergebnis;
      }
      
      Iterator<String> iterator = mapFont.keySet().iterator();
    
      while (iterator.hasNext())
      {
        fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
        
        if (fontUmsetzungTls.getPidFont().equals(pidFontOderPidFontUmsetzungTls))
        {
          ergebnis = fontUmsetzungTls.getCodeFont();
          
          return ergebnis;
        }
      }
    }
    
    return ergebnis;
  }
  
  /**
   * Abarbeitung der Autark-Versorgung aus dem Modell AnzeigenGlobal.
   * 
   * @param resultData AttributGruppe atg.anzeigeAutarkSoll.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeAutarkSoll(ResultData resultData, Long zeitpunktUebergeordnet)
  {
//  	System.out.println("AnzeigeTypUmsetzungTlsDefault.anzeigeAutarkSoll() " + resultData);
  	
    ResultData ergebnis = null;
    
    SystemObject objekt = resultData.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsAnzeige().get(objekt.getPid());

    if (anzeigeAktor == null)
      return null;
    
    if (resultData.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      return null;
    }
    else // Quelle vorhanden
    {
      if (resultData.hasData()) // Daten vorhanden
      {
        Data data = resultData.getData();
    
        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();
    
        long zeitpunkt = AnzeigenGlobalZwischenSchichtTls.getConnection().getTime();

        // BefehlsArt prfen
        
        int befehlsArt = AnzeigeSollBefehlsArt.autarkBefehlsArtBestimmen(data);
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt)
        {
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Grafik)
        {
          if (isKannFreieGrafik()) // Grafik-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Grafik-Ansteuerung");
            
            return null;
          }
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext)
        {
          if (isKannFreiText()) // Wechseltext-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Wechseltext-Ansteuerung");
            
            return null;
          }
        }
        else
        {
          m_debug.error("ATG fr Anzeige " + resultData.getObject().getPid() + " beinhaltet keinen Befehl");
          
          return null;
        }
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt) // reine Stellcode-Ansteuerung
        {
          HashMap mapWvzInhalt = null;
          
          ConfigurationObject objektAnzeigeInhalt = (ConfigurationObject)data.getItem("Autark").getReferenceValue("AnzeigeInhalt").getSystemObject();
          
          AttributeGroup atgWvzInhalt = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wvzInhalt");
          
          Data dataWvzInhalt = objektAnzeigeInhalt.getConfigurationData(atgWvzInhalt);
          String bildinhalt = dataWvzInhalt.getTextValue("Bildinhalt").getValueText();
          
          m_debug.finest("Bildinhalt " + bildinhalt);

          // DEs bestimmen
          
          int anzahlDes = anzeigeAktor.getDes().size();
          
          if (anzahlDes == 1)
          {
            mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsWvzInhalt();
            
            WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(bildinhalt);
  
            int code = -1;
            
            if (wvzInhaltUmsetzungTls != null)
            {
              code = wvzInhaltUmsetzungTls.getCode();
            }
            else
            {
            	if (bildinhalt.equals(AnzeigenGlobalZwischenSchichtTls.getBildinhaltBeibehalten()))
            	{
            		code = m_codeZustandBleibtErhalten;
            	}
            	else
            	{
            		m_debug.error("Objekt " + resultData.getObject().getPid() + " WvzInhalt " + bildinhalt + " nicht gefunden");
            	}
            }
            
            Set set = anzeigeAktor.getDes().keySet();
            
            Iterator it = set.iterator();
            
            String key = (String)it.next();
            
            SystemObject de = anzeigeAktor.getDes().get(key);
            
            int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

            for (int j = 0; j < anzahlGlobalUmsetzung; j++)
            {
              GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

              for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
              {
                GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                
                String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();
                
                if (asp.length() > 0)
                {
                  if ((atg.equals("atg.tlsWzgWvzGrundEinstellung" /*"atg.tlsWzgWvzStellZustand"*/)) ||
                      (atg.equals("atg.tlsWzgGrundEinstellung"/*"atg.tlsWzgStellZustand"*/)))
                  {
                    AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                    Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                    
                    Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                    if (atg.equals("atg.tlsWzgWvzGrundEinstellung" /*"atg.tlsWzgWvzStellZustand"*/))
                    {
                      dataErgebnis.getUnscaledValue("Code").set(code);

                      // BlinkenStatus
                      
                      String blinkenStatus = data.getItem("Autark").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                      dataErgebnis.getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
                      
                      // BlinkenDauer
                      
                      if (blinkenStatus.equals("ein"))
                      {
                        if (data.getItem("Autark").getScaledValue("Blinktakt").isState())
                        {
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                        }
                        else
                        {
                          double blinkenDauerAnzeige = data.getItem("Autark").getScaledValue("Blinktakt").doubleValue();
                          if (blinkenDauerAnzeige < 1.0)
                          {
                            blinkenDauerAnzeige = 1.0;
                          }
                          if (blinkenDauerAnzeige > 3.0)
                          {
                            blinkenDauerAnzeige = 3.0;
                          }
                          int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                          
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                        }
                      }
                      else
                      {
                        dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                      }
                      
                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    }
                    
                    if (atg.equals("atg.tlsWzgGrundEinstellung"/*"atg.tlsWzgStellZustand"*/))
                    {
                      dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                      
                      if ((anzeigePrinzip.equals("a")) || (anzeigePrinzip.equals("b")))
                      {
                        dataErgebnis.getArray("WVZAnzeige").setLength(1);
                        dataErgebnis.getArray("StellCode").setLength(0);
                      }
                      else
                      {
                        dataErgebnis.getArray("WVZAnzeige").setLength(0);
                        dataErgebnis.getArray("StellCode").setLength(1);
                      }
                      
                      dataErgebnis.getArray("TextDefinition").setLength(0);
                      dataErgebnis.getArray("GruppenDefinition").setLength(0);
                      
                      if ((anzeigePrinzip.equals("a")) || (anzeigePrinzip.equals("b")))
                      {
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getUnscaledValue("Code").set(code);
  
                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Autark").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
  
                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Autark").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Autark").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }
  
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");
                      }
                      else
                      {
                        dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(code);
                        
                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Autark").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
  
                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Autark").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Autark").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }
  
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");
                        
                        if (code == -1) // 'aus', Sonderfall bei WTA-C-Zeichen in BW
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(0);

                          dataErgebnis.getArray("TextDefinition").setLength(6);

                          int codeFont = fontUmsetzungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, anzeigeTypUmsetzungTls.getPidGrundFont());
                          int codeTextAusrichtung = textAusrichtungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, anzeigeTypUmsetzungTls.getGrundTextAusrichtung());
                          
//                          System.out.println("codeFont " + codeFont + " (" + anzeigeTypUmsetzungTls.getPidGrundFont() + ")");
//                          System.out.println("codeTextAusrichtung " + codeTextAusrichtung + " (" + anzeigeTypUmsetzungTls.getGrundTextAusrichtung() + ")");
                          
                          dataErgebnis.getArray("TextDefinition").getItem(0).getTextValue("Text").setText(String.format("%c", codeFont).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(1).getTextValue("Text").setText(String.format("%c", codeTextAusrichtung).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(2).getTextValue("Text").setText(String.format("%c", 0).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(3).getTextValue("Text").setText(String.format("%c", 0).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(4).getTextValue("Text").setText(String.format("%c", 0).toString());
                          dataErgebnis.getArray("TextDefinition").getItem(5).getTextValue("Text").setText(String.format("%c", 0).toString());                
                        }
                       }
                      
                      if (zeitpunkt < ((new Date()).getTime()-600000000))
                      {
                        zeitpunkt = zeitpunktUebergeordnet.longValue();
                      }
                      
                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    }
                    
                    break;
                  }
                }
              }
            }
          }
        }
        else
        {
          if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext) // reine Textansteuerung
          {
            ConfigurationObject wechseltext = (ConfigurationObject)data.getItem("Autark").getReferenceValue("Wechseltext").getSystemObject();

            Data dataWechseltext = null;
            
            if (wechseltext != null) // konfigurierter Wechseltext
            {
              AttributeGroup atgWechseltext = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wechseltext");
              dataWechseltext = wechseltext.getConfigurationData(atgWechseltext);
            }
            else // dynamischer Wechseltext
            {
              dataWechseltext = data.getItem("Autark").getItem("DynamischerWechseltext");
            }
            
            String pidFont = dataWechseltext.getReferenceValue("Font").getValueText();
            int codeFont = fontUmsetzungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidFont);

            String pidTextAusrichtung = dataWechseltext.getScaledValue("TextAusrichtung").getValueText();
            int codeTextAusrichtung = textAusrichtungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidTextAusrichtung);

            HashMap mapZeichen = null;
            
            HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
            
            if ((pidFont != null) && (mapFont != null))
            {
              TreeSet<String> treeset = new TreeSet<String>();
              treeset.addAll(mapFont.keySet());
            
              Iterator iterator = treeset.iterator();
            
              FontUmsetzungTls fontUmsetzungTls;
              
              while (iterator.hasNext())
              {
                fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
                
                if (fontUmsetzungTls.getPidFont().equals(pidFont))
                {
                  mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsZeichen();
                  
                  break;
                }
              }
            }

            // DEs bestimmen
              
            int anzahlDes = anzeigeAktor.getDes().size();
            
            if (anzahlDes == 1) // Wechseltext nur mit einer DE
            {
              Set set = anzeigeAktor.getDes().keySet();
              
              Iterator it = set.iterator();
              
              String key = (String)it.next();
              
              SystemObject de = anzeigeAktor.getDes().get(key);
              
              int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

              for (int j = 0; j < anzahlGlobalUmsetzung; j++)
              {
                GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

                for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
                {
                  GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                  
                  String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                  String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                  String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();

                  if (asp.length() > 0)
                  {
                    if ((atg.equals("atg.tlsWzgWechselText")) ||
                        (atg.equals("atg.tlsWzgGrundEinstellung" /*"atg.tlsWzgStellZustand"*/)))
                    {
                      AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                      Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                      
                      Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                      if (atg.equals("atg.tlsWzgWechselText"))
                      {
                        StringBuffer befehl = new StringBuffer();
                        
                        befehl.append(String.format("%c", codeFont));
                        befehl.append(String.format("%c", codeTextAusrichtung));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));
                                                
                        ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                        for (int i = 0; i < text.length; i++)
                        {
                          String pid = text[i].getSystemObject().getPid();
                          
                          if (mapZeichen == null)
                          {
                            continue;
                          }
                          
                          ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                          
                          if (zeichenUmsetzungTls != null)
                          {
                            befehl.append(String.format("%c", zeichenUmsetzungTls.getZeichencode()));
                          }
                          else
                          {
                            befehl.append(String.format("%c", 32)); // Leerzeichen als default
                          }
                        }

                        dataErgebnis.getTextValue("WVZTextZeichen").setText(befehl.toString());

                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Autark").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
                        
                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Autark").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }

                        if (zeitpunkt < ((new Date()).getTime()-600000000))
                        {
                          zeitpunkt = zeitpunktUebergeordnet.longValue();
                        }
                        
                        ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                      }
                      
                      if (atg.equals("atg.tlsWzgGrundEinstellung" /*"atg.tlsWzgStellZustand"*/))
                      {
                        dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                        dataErgebnis.getArray("WVZAnzeige").setLength(0);
                        dataErgebnis.getArray("StellCode").setLength(1);
                        dataErgebnis.getArray("TextDefinition").setLength(0);
                        dataErgebnis.getArray("GruppenDefinition").setLength(0);

                        dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(0);

                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Autark").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);

                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Autark").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Autark").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }

                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");

                        ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                        dataErgebnis.getArray("TextDefinition").setLength(6 + text.length);

                        int index = 0;
                        
//                        System.out.println("codeFont " + codeFont);
//                        System.out.println("codeTextAusrichtung " + codeTextAusrichtung);
                        
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeFont).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeTextAusrichtung).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());

                        int zeichen;
                        
                        for (int i = 0; i < text.length; i++)
                        {
                          String pid = text[i].getSystemObject().getPid();
                          
                          zeichen = 32; // Leerzeichen als default
                          
                          if (mapZeichen != null)
                          {
                            ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                            
                            if (zeichenUmsetzungTls != null)
                            {
                              zeichen = zeichenUmsetzungTls.getZeichencode();
                            }
                            else
                            {
                              zeichen = 32; // Leerzeichen als default
                            }
                          }

//                          System.out.println("- zeichen " + zeichen);
                          
                          dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", zeichen).toString());
                        }

                        if (zeitpunkt < ((new Date()).getTime()-600000000))
                        {
                          zeitpunkt = zeitpunktUebergeordnet.longValue();
                        }
                        
                        ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                      }
                      
                      break;
                    }
                  }
                }
              }
            }
            else // anzahlDes != 1
            {
              m_debug.error("Wechseltext nur mit einer DE mglich");
            }
          }
        }
      } 
      else // keine Daten vorhanden
      {
        return null;
      }
    }
    
    return ergebnis;
  }
    
  /**
   * Abarbeitung der TLS-WzgGrundEinstellung nach TLS 2002.
   * 
   * @param result Attributgruppe atg.tlsWzgGrundEinstellung.
   */
  public Data tlsWzgGrundEinstellung(ResultData result)
  {
//  	System.out.println("AnzeigeTypUmsetzungTlsBw1CZeichen.tlsWzgGrundEinstellung()");
    
    SystemObject de = result.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsDes().get(de.getPid());

    if (anzeigeAktor == null)
      return null;
    
    String anzeigePid = anzeigeAktor.getAnzeige().getPid();
    
    Data dataAnzeigeAutarkIst = AnzeigenGlobalZwischenSchichtTls.atgAnzeigeAutarkIstErstellen(AnzeigenGlobalZwischenSchichtTls.getConnection());
    
    m_debug.finer("tlsWzgGrundEinstellung DE " + de.getPid() + "\nAnzeige " + anzeigePid + "\nresult " + result + "\nData " + dataAnzeigeAutarkIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeAutarkIstInitialisieren(dataAnzeigeAutarkIst, result.getDataTime(), "Ja");
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();

        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();

        String prinzip = data.getScaledValue("AnzeigePrinzip").getValueText();
        
        if ((prinzip.equals("c")) || (prinzip.equals("d")))
        {
          if (data.getArray("StellCode").getLength() == 1)
          {
            int code = data.getArray("StellCode").getItem(0).getScaledValue("Code").intValue();

            if (code == 62) // Workaround fr Konfigurationsfehler
            	code = 9;
            
            m_debug.finest("Code " + code);

            if (code == 0) // freie TextDefinition
            {
              if (data.getArray("TextDefinition").getLength() < 7) // Wechseltext der nur aus Formatierung besteht als 'aus' annehmen
              {
              	code = -1; 
              	
                HashMap mapWvzInhalt = null;
                
                mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsCode();
                
                WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(code);
                
                if (wvzInhaltUmsetzungTls != null)
                {
                	dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setText(wvzInhaltUmsetzungTls.getPidWvzInhalt());
                }
                
                tlsWzgBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeAutarkIst, "Autark");
                
                dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");

                // Wechseltextinformationen lschen
                
                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("Wechseltext").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
              }
              else // Wechseltext bestimmen
              {
                // Ergebnis lschen    
                
              	dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
              	dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("Wechseltext").setSystemObject(null);
              	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
              	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
              	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
              
                String zeichenfolge = "";
  
                for (int i = 0; i < data.getArray("TextDefinition").getLength(); i++)
                {
                  zeichenfolge += data.getArray("TextDefinition").getItem(i).getTextValue("Text").getValueText();
                }
                
                int laengeInput = zeichenfolge.length();
                int laengeText = laengeInput-6;
          
                if (laengeInput >= 6)
                {

                  // Font aus Rckmeldung bestimmen, Grundfont nicht benutzen                  
//                  String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, zeichenfolge.codePointAt(0), pidGrundFontUmsetzungTls);
                  
                  int codeTextAusrichtung = zeichenfolge.codePointAt(0);
                  
                  // fr Tests Font ber ASCII 1, 2
                  
                  if (codeTextAusrichtung == 49)
                  {
                    codeTextAusrichtung = 128;
                  }
                  else if (codeTextAusrichtung == 50)
                  {
                    codeTextAusrichtung = 129;
                  }
                  
                  String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, codeTextAusrichtung, null);
          
                  FontUmsetzungTls fontUmsetzungTls = null;
      
                  if (pidFontUmsetzungTls != null)
                  {
                    HashMap mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
              
                    if (mapFont != null)
                      fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(pidFontUmsetzungTls);
                  }
      
                  HashMap mapZeichen = null;
            
                  if (fontUmsetzungTls != null)
                    mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsCode();
      
                  // hier mit Fehler abbrechen, wenn kein Font bestimmt !!      
                  
                  if ((mapZeichen == null) || (mapZeichen.size() == 0))
                  {
                  	tlsWzgBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeAutarkIst, "Autark");
                    dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");

                    m_debug.warning(de.getPid() + "Font ber Code " + codeTextAusrichtung + " nicht bestimmbar");

                    return dataAnzeigeAutarkIst;
                  }
                  
                  // Pid Font ber Umsetzung bestimmen
          
                  String pidFont = null;
            
                  if (fontUmsetzungTls != null)
                  {
                    pidFont = fontUmsetzungTls.getPidFont();
                  }
              
                  if (laengeText < 0)
                  {
                    laengeText = 0;
        
                    m_debug.warning(de.getPid() + "Wechseltext zu kurz (6 + Text) '" + zeichenfolge + "'");
                
                    return null;
                  }
              
                  if (pidFont != null)
                  {
                  	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setText(pidFont);
                  }

                  // Textausrichtung aus Rckmeldung bestimmen
      
                  codeTextAusrichtung = zeichenfolge.codePointAt(1);
                  
                  // fr Tests Ausrichtung ber ASCII 1, 2, 3
                  
                  if (codeTextAusrichtung == 49)
                  {
                    codeTextAusrichtung = 133;
                  }
                  else if (codeTextAusrichtung == 50)
                  {
                    codeTextAusrichtung = 134;
                  }
                  else if (codeTextAusrichtung == 51)
                  {
                    codeTextAusrichtung = 135;
                  }

                  String textAusrichtung = textAusrichtungAusTlsBestimmen(anzeigeTypUmsetzungTls, codeTextAusrichtung);
                  
                  System.out.println("codeTextAusrichtung " + codeTextAusrichtung + " textAusrichtung " + textAusrichtung);
              
                  dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").setText(textAusrichtung);
            
                  // Text        
            
                  dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(laengeText);
      
                  for (int i = 6; i < zeichenfolge.length(); i++)
                  {
                    m_debug.finest("Zeichen " + i + " " + zeichenfolge.codePointAt(i));
          
                    int codeZeichen = zeichenfolge.codePointAt(i);
          
                    ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(codeZeichen);
                  
                    if (zeichenUmsetzungTls != null)
                    	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setText(zeichenUmsetzungTls.getPidZeichen());
                    else
                    	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setSystemObject(null);
                  }
                  
                  dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");
                }
                else // fehlerhafte Lnge, wenn grer als 0, wird mindestens 6 erwartet
                {
                  dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");
                }
                
                tlsWzgBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeAutarkIst, "Autark");
              }
            }
            else // code != 0
            {
            	if (code == m_codeZustandBleibtErhalten)
            	{
                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setText(AnzeigenGlobalZwischenSchichtTls.getWvzInhaltBeibehalten());
                dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");
                
                tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");
                
                // Wechseltextinformationen lschen
                
                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("Wechseltext").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
            	}
            	else
            	{
	              HashMap mapWvzInhalt = null;
	              
	              mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsCode();
	              
	              WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(code);
	              
	              if (wvzInhaltUmsetzungTls != null)
	              {
	                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setText(wvzInhaltUmsetzungTls.getPidWvzInhalt());
	                dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");
	              }
	              else
	              {
	                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
	                dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");
	              }
	              
	              tlsWzgBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeAutarkIst, "Autark");
	              
	              // Wechseltextinformationen lschen
	              
	              dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("Wechseltext").setSystemObject(null);
	              dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
	              dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
	              dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
            	}
            }
          }
          else
          {
            tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");

            dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Ja");
          }
        }
      }
      else // keine Daten vorhanden
      {
        tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");

        dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Ja");
      }
    }
    
    return dataAnzeigeAutarkIst;
  }  
  
  
}
